#ifdef HAVE_XDR

/*****
*
* Copyright (C) 1998, 1999, 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <assert.h>
#include <rpc/types.h>
#include <rpc/xdr.h>

#include "packet.h"
#include "common.h"
#include "alert.h"
#include "alert-prv.h"
#include "rxdr.h"

#define xdr_u_int8(xdrs, var) xdr_u_char(xdrs, var)
#define xdr_u_int16(xdrs, var) xdr_u_short(xdrs, var)
#define xdr_u_int32(xdrs, var) xdr_u_int(xdrs, var)



/*
 * Translate an Ip header into XDR data
 */

static int xdr_ip(XDR *xdrs, iphdr_t *ip)
{
	int ret;
           
	ret = xdr_u_int8(xdrs, &ip->ip_tos);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(ip_tos).\n");
		return FALSE;
	}
        
        ret = xdr_u_short(xdrs, &ip->ip_len);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ip_len).\n");
		return FALSE;
	}

	ret = xdr_u_short(xdrs, &ip->ip_id);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ip_id).\n");
		return FALSE;
	}
	ret = xdr_u_short(xdrs, &ip->ip_off);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ip_off).\n");
		return FALSE;
	}
	ret = xdr_u_int8(xdrs, &ip->ip_ttl);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(ip_ttl).\n");
		return FALSE;
	}
	ret = xdr_u_int8(xdrs, &ip->ip_p);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(ip_p).\n");
		return FALSE;
	}
	ret = xdr_u_short(xdrs, &ip->ip_sum);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ip_sum).\n");
		return FALSE;
	}
	ret = xdr_u_int32(xdrs, &ip->ip_src.s_addr);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int32(ip_src.s_addr).\n");
		return FALSE;
	}
	ret = xdr_u_int32(xdrs, &ip->ip_dst.s_addr);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int32(ip_dst.s_addr).\n");
		return FALSE;
	}

	return TRUE;
}



static int xdr_tcpip_opts(XDR *xdrs, packet_t *p) 
{
        int ret;

        if ( p->len > 0 ) {                
                ret = xdr_bytes(xdrs, (char **) &p->p.opts, &p->len, MAX_OPTS_LEN);
                if ( ret == FALSE ) {
                        log(LOG_ERR, "xdr_bytes(opts).\n");
                        return FALSE;
                }
        }

        return TRUE;
}



static int xdr_udphdr(XDR * xdrs, udphdr_t *udp)
{
	int ret;
        
	ret = xdr_u_int16(xdrs, &udp->uh_sport);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(uh_sport).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &udp->uh_dport);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(uh_dport).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &udp->uh_ulen);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(uh_ulen).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &udp->uh_sum);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(uh_sum).\n");
		return FALSE;
	}
        
	return TRUE;
}

static int xdr_tcphdr(XDR * xdrs, tcphdr_t *tcp_hdr)
{
	int ret;
           
        ret = xdr_u_int16(xdrs, &tcp_hdr->th_sport);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(th_sport).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &tcp_hdr->th_dport);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(th_dport).\n");
		return FALSE;
	}
	ret = xdr_u_int32(xdrs, &tcp_hdr->th_seq);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int32(th_seq).\n");
		return FALSE;
	}
	ret = xdr_u_int32(xdrs, &tcp_hdr->th_ack);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int32(th_ack).\n");
		return FALSE;
	}
	ret = xdr_u_int8(xdrs, &tcp_hdr->th_flags);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(th_flags).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &tcp_hdr->th_win);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(th_win).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &tcp_hdr->th_sum);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(th_sum).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &tcp_hdr->th_urp);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(th_urp).\n");
		return FALSE;
	}

        return TRUE;
}

static int xdr_icmphdr(XDR * xdrs, icmphdr_t *icmp)
{
	int ret;

#warning FIXME: xdr_icmphdr is not finished !

	ret = xdr_u_int8(xdrs, &icmp->icmp_type);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(icmp_type).\n");
		return FALSE;
	}
	ret = xdr_u_int8(xdrs, &icmp->icmp_code);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(icmp_code).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &icmp->icmp_cksum);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(icmp_cksum).\n");
		return FALSE;
	}

	return TRUE;
}

static int xdr_igmphdr(XDR * xdrs, igmphdr_t *igmp)
{
	int ret;

	ret = xdr_u_int8(xdrs, &igmp->igmp_type);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(igmp_type).\n");
		return FALSE;
	}
	ret = xdr_u_int8(xdrs, &igmp->igmp_code);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int8(igmp_code).\n");
		return FALSE;
	}
	ret = xdr_u_int16(xdrs, &igmp->igmp_cksum);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int16(igmp_cksum).\n");
		return FALSE;
	}
	ret = xdr_u_int32(xdrs, &igmp->igmp_group.s_addr);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_int32(igmp_group.s_addr).\n");
		return FALSE;
	}
        
	return TRUE;
}

static int xdr_arphdr(XDR * xdrs, etherarphdr_t *arp)
{
	int ret, i;
        
	ret = xdr_u_short(xdrs, &arp->arp_hrd);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ar_hrd).\n");
		return FALSE;
	}
	ret = xdr_u_short(xdrs, &arp->arp_pro);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ar_pro).\n");
		return FALSE;
	}
	ret = xdr_u_char(xdrs, &arp->arp_hln);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_char(ar_hln).\n");
		return FALSE;
	}
	ret = xdr_u_char(xdrs, &arp->arp_pln);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_char(ar_pln).\n");
		return FALSE;
	}
	ret = xdr_u_short(xdrs, &arp->arp_op);
	if (ret == FALSE) {
		log(LOG_ERR, "xdr_u_short(ar_op).\n");
		return FALSE;
	}


        for (i = 0; i < ETH_ALEN; i++) {
                ret = xdr_u_int8(xdrs, &arp->arp_sha[i]);
                if (!ret) {
                        log(LOG_ERR, "xdr_u_int8(ether_hdr->ether_dhost[i]).\n");
                        return FALSE;
                }
                ret = xdr_u_int8(xdrs, &arp->arp_tha[i]);
                if (!ret) {
                        log(LOG_ERR, "xdr_u_int8(ether_hdr->ether_shost[i]).\n");
                        return FALSE;
                }
        }

        for (i = 0; i < 4; i++) {
                ret = xdr_u_int8(xdrs, &arp->arp_spa[i]);
                if (!ret) {
                        log(LOG_ERR, "xdr_u_int8(ether_hdr->ether_dhost[i]).\n");
                        return FALSE;
                }
                ret = xdr_u_int8(xdrs, &arp->arp_tpa[i]);
                if (!ret) {
                        log(LOG_ERR, "xdr_u_int8(ether_hdr->ether_shost[i]).\n");
                        return FALSE;
                }
        }

	return TRUE;
}

static int xdr_ether_header(XDR * xdrs, etherhdr_t *ether)
{
	int ret, i;

        for (i = 0; i < ETH_ALEN; i++) {
                ret = xdr_u_int8(xdrs, &ether->ether_dhost[i]);
                if (!ret) {
                        log(LOG_ERR, "xdr_u_int8(ether_hdr->ether_dhost[i]).\n");
                        return FALSE;
                }
                ret = xdr_u_int8(xdrs, &ether->ether_shost[i]);
                if (!ret) {
                        log(LOG_ERR, "xdr_u_int8(ether_hdr->ether_shost[i]).\n");
                        return FALSE;
                }
        }

        ret = xdr_u_int16(xdrs, &ether->ether_type);
	if (!ret) {
		log(LOG_ERR, "xdr_u_int16(ether_hdr->ether_type).\n");
		return FALSE;
	}

	return TRUE;
}

static int xdr_pktdata(XDR * xdrs, packet_t *p)
{
	int ret;
        
	ret = xdr_u_int(xdrs, (unsigned *) &p->len);
	if (!ret) {
		log(LOG_ERR, "xdr_u_int(__pkt_data.size).\n");
		return FALSE;
	}

        ret = xdr_bytes(xdrs, (char **) &p->p.data, &p->len, p->len);
	if (!ret) {
		log(LOG_ERR, "xdr_bytes(__pkt_data.data).\n");
		return FALSE;
	}
	return TRUE;
}

static int xdr_packet_encode(XDR * xdrs, packet_t *packet)
{
        int i = 0, ret;

        do {
                ret = xdr_enum(xdrs, (enum_t *) &packet[i].proto);
                if (!ret) {
                        log(LOG_ERR, "xdr_enum(&packet[%d].proto = %d).\n", i, packet[i].proto);
                        return FALSE;
                }

                ret = xdr_int(xdrs, &packet[i].len);
                if ( ! ret ) {
                        log(LOG_ERR, "error converting len member to xdr.\n");
                        return FALSE;
                }
                
                if ( packet[i].proto == p_end ) 
                        break;
                
                switch (packet[i].proto) {
                case p_ether:                        
                        ret = xdr_ether_header(xdrs, packet[i].p.ether_hdr);
                        break;
                case p_ip:
                case p_ipencap:
                        ret = xdr_ip(xdrs, packet[i].p.ip);
                        break;

                case p_ipopts:
                        xdr_tcpip_opts(xdrs, &packet[i]);
                        break;
                        
                case p_tcp:
                        ret = xdr_tcphdr(xdrs, packet[i].p.tcp);
                        break;

                case p_tcpopts:
                        xdr_tcpip_opts(xdrs, &packet[i]);
                        break;
                        
                case p_udp:
                        ret = xdr_udphdr(xdrs, packet[i].p.udp_hdr);
                        break;
                case p_icmp:
                        ret = xdr_icmphdr(xdrs, packet[i].p.icmp_hdr);
                        break;
                case p_igmp:
                        ret = xdr_igmphdr(xdrs, packet[i].p.igmp_hdr);
                        break;
                case p_rarp:
                case p_arp:
                        ret = xdr_arphdr(xdrs, packet[i].p.arp_hdr);
                        break;
                case p_data:
                        ret = xdr_pktdata(xdrs, &packet[i]);
                        break;
                default:
                        break;
                }

                i++;
                
        } while ( ret != FALSE);

        assert(packet[(i == MAX_PKTDEPTH) ? i - 1 : i].proto == p_end);
        
        if ( ret == FALSE ) {
                log(LOG_ERR, "The received report seem to be corrupted.\n");
                return FALSE;
        }
        
        return TRUE;
}

static int xdr_packet_decode(XDR * xdrs, packet_t *packet)
{
        int i = 0, j = 0, ret;
        static etherhdr_t ether;
        static iphdr_t ip, ipencap[20];
        static tcphdr_t tcp;
        static udphdr_t udp;
        static etherarphdr_t arp, rarp;
        static icmphdr_t icmp;
        static igmphdr_t igmp;
        
        do {
                ret = xdr_enum(xdrs, (enum_t *) &packet[i].proto);
                if (!ret) {
                        log(LOG_ERR, "xdr_enum(&packet[%d].proto = %d).\n", i, packet[i].proto);
                        return FALSE;
                }

                ret = xdr_int(xdrs, &packet[i].len);
                if ( ! ret ) {
                        log(LOG_ERR, "error converting len member to xdr.\n");
                        return FALSE;
                }

                if ( packet[i].proto == p_end )
                        break;

                switch (packet[i].proto) {
                case p_ether:
                        ret = xdr_ether_header(xdrs, &ether);
                        packet[i].p.ether_hdr = &ether;
                        break;
                case p_ip:
                        ret = xdr_ip(xdrs, &ip);
                        packet[i].p.ip = &ip;
                        break;     
                case p_ipencap:
                        ret = xdr_ip(xdrs, &ipencap[j]);
                        packet[i].p.ip = &ipencap[j++];
                        break;
                case p_ipopts:
                        packet[i].p.opts = NULL;
                        xdr_tcpip_opts(xdrs, &packet[i]);
                        break;
                case p_tcp:
                        ret = xdr_tcphdr(xdrs, &tcp);
                        packet[i].p.tcp = &tcp;
                        break;
                case p_tcpopts:
                        packet[i].p.opts = NULL;
                        xdr_tcpip_opts(xdrs, &packet[i]);
                        break;
                case p_udp:
                        ret = xdr_udphdr(xdrs, &udp);
                        packet[i].p.udp_hdr = &udp;
                        break;
                case p_icmp:
                        ret = xdr_icmphdr(xdrs, &icmp);
                        packet[i].p.icmp_hdr = &icmp;
                        break;
                case p_igmp:
                        ret = xdr_igmphdr(xdrs, &igmp);
                        packet[i].p.igmp_hdr = &igmp;
                        break;
                case p_rarp:
                        ret = xdr_arphdr(xdrs, &rarp);
                        packet[i].p.arp_hdr = &rarp;
                case p_arp:
                        ret = xdr_arphdr(xdrs, &arp);
                        packet[i].p.arp_hdr = &arp;
                        break;
                case p_data:
                        packet[i].p.data = NULL;
                        ret = xdr_pktdata(xdrs, &packet[i]);
                            //packet[i].p.pkt_data = &data;
                        break;
                default:
                        break;
                }

                i++;
                
        } while ( ret != FALSE && i < MAX_PKTDEPTH );

        assert(packet[(i == MAX_PKTDEPTH) ? i - 1 : i].proto == p_end);
        
        if ( ret == FALSE ) {
                log(LOG_ERR, "The received report seem to be corrupted.\n");
                return FALSE;
        }
        return TRUE;
}


static int xdr_alert(XDR *xdrs, alert_t *alert) 
{
        int ret;
        char *ptr;
        unsigned long l;
        
        ret = xdr_wrapstring(xdrs, &plugin_name(alert_plugin(alert)));
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%s).\n", plugin_name(alert_plugin(alert)));
		return FALSE;
	}

        ret = xdr_wrapstring(xdrs, &plugin_author(alert_plugin(alert)));
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%s).\n", plugin_author(alert_plugin(alert)));
		return FALSE;
	}

        ret = xdr_wrapstring(xdrs, &plugin_contact(alert_plugin(alert)));
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%s).\n", plugin_contact(alert_plugin(alert)));
		return FALSE;
	}
        
	ret = xdr_wrapstring(xdrs, &plugin_desc(alert_plugin(alert)));
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%s).\n", plugin_desc(alert_plugin(alert)));
		return FALSE;
	}

        ret = xdr_wrapstring(xdrs, &alert_quickmsg(alert));
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%s).\n", alert_quickmsg(alert));
		return FALSE;
	}
        
	ret = xdr_enum(xdrs, (enum_t *) &alert_kind(alert));
	if (!ret) {
		log(LOG_ERR, "xdr_enum(%d).\n", alert_kind(alert));
		return FALSE;
	}

        ret = xdr_u_int(xdrs, &alert_count(alert));
        if (!ret) {
                log(LOG_ERR, "xdr_int(%d).\n", alert_count(alert));
                return FALSE;
        }

        ptr = alert_message(alert);
	ret = xdr_wrapstring(xdrs, &ptr);
	if (!ret) {
                log(LOG_ERR, "xdr_wrapstring(%s).\n", alert_message(alert));
		return FALSE;
	}

        l = (unsigned long) alert_time_start(alert);
        ret = xdr_u_long(xdrs, &l);
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%ld).\n", l);
		return FALSE;
	}

        l = (unsigned long) alert_time_end(alert);
        ret = xdr_u_long(xdrs, &l);
	if (!ret) {
		log(LOG_ERR, "xdr_wrapstring(%ld).\n", l);
		return FALSE;
	}

        return TRUE;
}



/*
 * This function support encoding (to) and decoding (from) an
 * XDR stream (XDRS) :
 */
int rxdr_encode(alert_t *alert, XDR *xdrs)
{
	int ret;

        ret = xdr_alert(xdrs, alert);
	if (!ret) 
		return -1;

        ret = xdr_packet_encode(xdrs, alert_packet(alert)->packet);
        if (!ret) 
                return -1;
        
        ret = xdrrec_endofrecord(xdrs, TRUE);
        if (!ret) {
                log(LOG_ERR, "error setting end of record on XDR stream.\n");
                return -1;
        }
        
	return 0;
}


int rxdr_decode(alert_t *alert, XDR *xdrs)
{
        int ret;
        
        ret = xdr_alert(xdrs, alert);
	if (!ret) 
		return -1;

        ret = xdr_packet_decode(xdrs, alert_packet(alert)->packet);
        if (!ret) 
                return -1;
        
	return 0;
}

#endif

