/*****
*
* Copyright (C) 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>

#include "detect.h"


static plugin_detect_t plugin;
static subscribtion_t subscribtion[] = {
        { p_all, NULL },
        { p_end, NULL },
};



static void debug_run(packet_container_t *packet) 
{
        plugin_do_report((plugin_generic_t *) &plugin, packet, normal,
                         "Debug reporting",
                         "This is a debuging message by the debug plugin");

}



static void print_help(const char *optarg) 
{
        fprintf(stderr, "Usage for %s :\n", plugin.name);
        fprintf(stderr, "\t -e --enable (default is disabled)\n\n");
}


static int enable = 0;


static void set_enable(const char *optarg) 
{
        enable = 1;
}


int plugin_init(unsigned int id)
{
        plugin_option_t opts[] = {
                { "enable", no_argument, NULL, 'e', set_enable },
                { "help", no_argument, NULL, 'h', print_help   },
                { 0, 0, 0, 0 },
        };
        
        plugin_set_name(&plugin, "Debug");        
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "Send an alert for each packet.");
        plugin_set_running_func(&plugin, debug_run);
        plugin_set_subscribtion(&plugin, subscribtion);

        plugin_config_get((plugin_generic_t *) &plugin, opts, PRELUDE_CONF);

        if ( ! enable )
                return -1;
        
        return plugin_register((plugin_generic_t *) &plugin);
}
