/*****
*
* Copyright (C) 2001 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Written by Yoann Vandoorselaere <yoann@mandrakesoft.com>
*
*****/


/*
 * A lot of the code in this plugin is based on the Snort
 * http_decode preprocessor, Copyright (C) 1998,1999,2000,
 * 2001 Martin Roesch <roesch@clark.net>
 */


#include <stdio.h>
#include <ctype.h>

#include "protocol.h"
#include "timer.h"
#include "alert.h"
#include "alert-prv.h"
#include "rqueue.h"


/*
 * This code is based on the Snort http_decode preprocessor.
 * Lot of the code here was grabbed from this place.
 */


typedef struct {
        unsigned char *uri;
        int length;
} http_uri_t;



static int http_plugin_id = 0;
static int check_iis_unicode = -1;
static int check_cgi_null = -1;
static plugin_protocol_t plugin;
static http_uri_t http_uri;
static port_list_t port_list;


#ifdef DEBUG

static void dump_uri(http_uri_t *ptr) 
{
	int i;

	printf("length=%d fstring=\"", ptr->length);
	
	for ( i = 0; i < ptr->length; i++ )
		if ( isprint(ptr->uri[i]) )
			putchar(ptr->uri[i]);
	
	printf("\"\n");
}

#endif



static int match_uricontent(packet_container_t *pc, void *data) 
{
        http_uri_t *ptr;
        
        if ( pc->protocol_plugin_id != http_plugin_id )
                return -1;
        
        ptr = pc->protocol_plugin_data;
        if ( ! ptr ) {
		http_uri.length = pc->packet[pc->application_layer_depth].len;
		http_uri.uri = pc->packet[pc->application_layer_depth].p.data;
		ptr = &http_uri;
	}
	
        return signature_engine_match_content(ptr->uri, ptr->length, data);
}




/*
 * Function: nibble(char)
 *
 * Purpose: converts a hex or 32base character into a value 
 *          in the range 0..15 (or 0..31)
 *
 * Arguments: what => the character in question
 *
 * Returns: The converted character or -1 if the character is
 *          not 16base (hex) or 32base (?)
 *
 */
static int nibble(char what)
{
        if ( isdigit((int)what) )
                return what - '0';

        if ( isalpha((int)what) )
                return toupper((int)what) - 'A' + 10;

        return -1;
}
        


static void check_for_attack(packet_container_t *packet, int unicode) 
{
        /*
         * test for : "/", "\", ".".
         */
        if ( ((unicode == 0x2f) || (unicode == 0x5c) || (unicode == 0x2e)) && check_iis_unicode ) {
                plugin_do_report((plugin_generic_t *) &plugin, packet,
                                 maybe_not_reliable, "ISS Unicode attack",
                                 "ISS Unicode attack detected");
        }
        
        if ( unicode == 0 && check_cgi_null ) {
                plugin_do_report((plugin_generic_t *) &plugin, packet,
                                 maybe_not_reliable, "CGI Null byte attack",
                                 "CGI Null byte attack detected");
        }
}



static int decode_http_packet(packet_container_t *packet, unsigned char *data, int psize) 
{
        char *url, *index, *end;
        int unicode, temp, temp2 = 0, temp3, first_space_found = 0;
        
        http_uri.length = 0;
        http_uri.uri = NULL;
        
        index = url = (char *) data;
        end = (char *) data + psize;
        
        while ( index < end ) {
                
                /*
                 * if it's potentially "escaped" and not too close to the 
                 * end of the payload 
                 */
                if ( (*index == '%') && (index < end - 2) && ! http_uri.length ) {
                        unicode = -1;
                        temp = (((nibble((int) *(index + 1)) << 4) + nibble((int) * (index + 2))) & 0xff);

                        
                        /*
                         * 0xbf - 0xe0 range
                         */
                        if ( (temp > 0xbf) && (temp < 0xe0) && (*(index + 3) == '%') && (index < (end - 5)) ) {
                                temp2 = (((nibble((int) * (index + 4) ) <<4) + nibble((int)*(index+5))) & 0xff);
                                unicode = (temp & 0x1f) << 6 | (temp2 & 0x3f);
                                index += 6;
                                url += 2;
                                psize -= 4;
                        }
                        
                        /* 0xdf - 0xf0 range */
                        else if ((temp > 0xdf) && (temp < 0xf0) && (*(index + 3) == '%') && (index < (end - 5)) ) {
                                temp2 = (((nibble((int)*(index+4))<<4) + nibble((int)*(index+5))) & 0xff);
                                index += 6;
                                url += 2;
                                psize -= 4;
                              
				if ( (*(index) == '%') && (index < (end - 2)) ) {
					temp3 = (((nibble((int)*(index+1))<<4) + nibble((int)*(index+2))) & 0xff);
					unicode = (temp & 0x0f) << 4 | (temp2 & 0x3f) << 6| (temp3 & 0x3f);
					index += 3;
					url++;
					psize -= 2;
				}
			}
                
                        /*
                         * anything else that is valid hex
                         */
                        else if ( isxdigit((int) * (index + 1)) && isxdigit((int) * (index + 2)) ) {
                                *url = (char) 0x000000FF & temp;
                                index += 3;
                                url++;
                                psize -=2;
                        }
                        
                        else {
                                *url = *index;
                                
                                if ( first_space_found && *url != ' ' && ! http_uri.uri )
                                        http_uri.uri = index;
                                
                                url++;
                                index++;
                        }
                        
                        check_for_attack(packet, unicode);
                }
        
        
                else {
                        *url = *index;
                        
                        if ( *url == ' ' && ! first_space_found ) 
                                first_space_found = 1;

                        /*
                         * Start filling the URI.
                         */
                        if ( first_space_found && *url != ' ' && ! http_uri.uri )
                                http_uri.uri = index;

                        /*
                         * Stop filling the URI.
                         */
                        if ( ! http_uri.length && http_uri.uri && (*url == ' ' || *url == '\n') ) {
                                char *foo = http_uri.uri;
                                http_uri.length = (url - foo);
                        }
                        
                        url++;
                        index++;
                }
        }

        
        /* 
         * the length might not get set for things like gfx, so we should
         * set it on the off hand that a uri pointer was set
         */
        if ( http_uri.length ) {
                /*
		 * This mark the packet as having being modified by this protocol plugin.
		 * It'll be used later in the match function.
		 */ 
		packet->protocol_plugin_id = http_plugin_id;
                packet->protocol_plugin_data = &http_uri;
                
                /*
		 * Payload not modified.
		 */ 
                return 0;
        }

        return -1;
}




static int http_decode(packet_container_t *pc, unsigned char *data, int psize) 
{
        int ret;
        int depth = pc->transport_layer_depth;
        unsigned int dport;
        
        if ( depth < 0 || pc->packet[depth].proto != p_tcp )
                return -1;

        dport = ntohs(pc->packet[depth].p.tcp->th_dport);
        
        ret = protocol_plugin_is_port_ok(&port_list, dport);
        if ( ret < 0 )
                return -1;

        return decode_http_packet(pc, data, psize);
}



static int parse_uricontent(char *str, rules_t **rules) 
{
        int ret;
        
        ret = signature_parser_parse_content(str, rules);
        if ( ret < 0 )
                return -1;
        
        (*rules)->rule->leaf_match->leaf_match = &match_uricontent;

        return 0;
}



static void print_help(const char *optarg) 
{
        fprintf(stderr, "Usage for %s :\n", plugin.name);
        fprintf(stderr, "\t -p --port-list  List of port this plugin should look at.\n");
        fprintf(stderr, "\t -c --disable-cginull  Disable cgi NULL check.\n");
        fprintf(stderr, "\t -u --disable-unicode  Disable ISS Unicode attack check.\n\n");
}



static void setup_default_port_list(void) 
{
        protocol_plugin_add_port_to_list(&port_list, 80);
        protocol_plugin_add_port_to_list(&port_list, 8080);
}



static void set_port_list(const char *optarg) 
{
        protocol_plugin_add_string_port_to_list(&port_list, optarg);
}



static void set_disable_cginull(const char *optarg) 
{
        check_cgi_null = 0;
}



static void set_disable_unicode(const char *optarg) 
{
        check_iis_unicode = 0;
}




int plugin_init(unsigned int id)
{
        plugin_option_t opts[] = {
                { "port-list", required_argument, NULL, 'p', set_port_list       },
                { "disable-cginull", no_argument, NULL, 'c', set_disable_cginull },
                { "disable-unicode", no_argument, NULL, 'u', set_disable_unicode },
                { "help", no_argument, NULL, 'h', print_help                     },
                { 0, 0, 0, 0 },
        };

        http_plugin_id = id;
        
        plugin_set_name(&plugin, "HttpMod");        
        plugin_set_protocol(&plugin, "http");
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "Snort based http decode plugin.");
        plugin_set_running_func(&plugin, http_decode);
        
        plugin_config_get((plugin_generic_t *)&plugin, opts, PRELUDE_CONF);

        if ( protocol_plugin_is_port_list_empty(&port_list) )
                setup_default_port_list();
        
        signature_parser_add_one_arg_key("uricontent", &parse_uricontent);
        
        return plugin_register((plugin_generic_t *) &plugin);
}

