/*****
*
* Copyright (C) 2001 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Written by Yoann Vandoorselaere <yoann@mandrakesoft.com>
*
*****/


#include <stdio.h>
#include <stdlib.h>

#include "protocol.h"
#include "rpc-prv.h"


int rpc_plugin_id;
port_list_t port_list;


static int add_rpc_rule(rpc_test_t *ptr, rules_t **rules) 
{
        int ret;
        rule_t *rule;

        rule = make_new_rule(0, NULL);
        if ( ! rule )
                return -1;
        
        ret = add_rule_leaf_match(rule, 1, ptr, &match_rpc_rule);
        if ( ret < 0 )
                return -1;
        
        *rules = make_new_rules(rule, NULL);
        if ( ! *rules )
                return -1;
        
        return 0;
}



static int parse_rpc_rule(parameters_t *plist, rules_t **rules)
{
        int i;
        rpc_test_t *ptr;
        
        ptr = calloc(1, sizeof(rpc_test_t));
        if ( ! ptr ) {
                log(LOG_ERR, "memory exhausted.\n");
                return -1;
        }

        for ( i = 0; plist != NULL; plist = plist->next ) {

                /*
                 * This parameter is empty.
                 */
                if ( *plist->str == '*' ) {
                        i++;
                        continue;
                }                       

                if ( i == 0 ) {
                        ptr->procedure = strtoul(plist->str, NULL, 0);
                        ptr->flags |= RPC_CHECK_PROCEDURE;
                }
                
                else if ( i == 1 ) {
                        ptr->version = strtoul(plist->str, NULL, 0);
                        ptr->flags |= RPC_CHECK_VERSION;
                }
                
                else if ( i == 2 ) {
                        ptr->program = strtoul(plist->str, NULL, 0);
                        ptr->flags |= RPC_CHECK_PROGRAM;
                }

                i++;
        }

        if ( ptr->flags )
                return add_rpc_rule(ptr, rules);
        
        free(ptr);
        return -1;
}




static plugin_protocol_t plugin;


static void print_help(const char *optarg) 
{
        fprintf(stderr, "Usage for %s :\n", plugin.name);
        fprintf(stderr, "\t -p --port-list  List of port this plugin should look at.\n\n");
}



static void setup_default_port_list(void) 
{
	protocol_plugin_add_port_to_list(&port_list, 111);
}



static void set_port(const char *optarg) 
{
        protocol_plugin_add_port_to_list(&port_list, atoi(optarg));
}



int plugin_init(unsigned int id)
{
        plugin_option_t opts[] = {
                { "port-list", required_argument, NULL, 'p', set_port },
                { "help", no_argument, NULL, 'h', print_help          },
                { 0, 0, 0, 0 },
        };

	rpc_plugin_id = id;
        protocol_plugin_init_port_list(&port_list);
        
        plugin_set_name(&plugin, "RpcMod");        
        plugin_set_protocol(&plugin, "rpc");
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "Decode the RPC protocol and provide rpc key for the signature engine.");
        plugin_set_running_func(&plugin, decode_rpc);

        plugin_config_get((plugin_generic_t *)&plugin, opts, PRELUDE_CONF);

        if ( protocol_plugin_is_port_list_empty(&port_list) )
                setup_default_port_list();
        
        signature_parser_add_multiple_args_key("rpc", &parse_rpc_rule);
        
        return plugin_register((plugin_generic_t *) &plugin);
}



